
// Take care of the deletion of a AIDA::Loader
// (in particular if allocated on the stack).
// Be sure to no more use code related to the Loader
// after the closing of it (done by the ~AIDA::Loader).

// Take care of the C functions declared to ::atexit in a Loader.
// On some system (RedHat-6.x) the dlclose does not call them !
// Instead they are called at the (final) exit. Then crashes can
// happen if someone dlclose the Loader before calling exit (or returning
// from the main).

// this :
#include "Loader.h"

#include <stdlib.h>

#ifdef WIN32
#include <windows.h>
#endif

#if defined(__alpha) || defined(__linux) || defined(__sun) || defined(__APPLE__)
#define DLOPEN
#endif

#ifdef DLOPEN
#include <dlfcn.h>
#endif

#include <iostream>

//////////////////////////////////////////////////////////////////////////////
AIDA::Loader::Loader(
 const std::string& aName
,bool aQuiet
,bool aVerbose
)
:fModule(0)
,fName(aName)
,fVerbose(aVerbose)
//////////////////////////////////////////////////////////////////////////////
// aName is the full name (path + libname) or could be simply
// the DLL name if access path are specified.
// Access path are :
// - PATH on Windows.
// - LD_LIBRARY_PATH on Linuxes.
// - DYLD_LIBRARY_PATH on MacOSX.
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(!open()) {
    if(!aQuiet) {
      std::cerr << "AIDA::Loader::Loader :" 
                << " can't load " << fName << " library." 
                << std::endl;
      std::cerr << "AIDA::Loader::Loader :" 
                << " error  : " << fError 
                << std::endl;
    }
  }
}
//////////////////////////////////////////////////////////////////////////////
AIDA::Loader::~Loader(
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if (fModule) {
    if(fVerbose) {
      std::cout << "AIDA::Loader::~Loader :" 
                << " close \"" << fName << "\"..." 
                << std::endl;
    }
#ifdef DLOPEN
    if(::dlclose(fModule)) {
      std::cerr << "AIDA::Loader::~Loader :" 
                << " error  : " << ::dlerror() 
                << std::endl;
    }
#endif
#ifdef WIN32
    ::FreeLibrary((HMODULE)fModule);
#endif
    if(fVerbose) {
      std::cout << "AIDA::Loader::~Loader :" 
                << " close \"" << fName << "\" done." 
                << std::endl;
    }
  }
  fModule = 0;
}
//////////////////////////////////////////////////////////////////////////////
bool AIDA::Loader::open(
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  fError = "";
#ifdef DLOPEN
  int flags = RTLD_NOW | RTLD_GLOBAL;
  if(fVerbose) {
    std::cout << "AIDA::Loader::open :" 
              << " open \"" << fName 
              << "\" (in mode RTLD_NOW | RTLD_GLOBAL)..." 
              << std::endl;
  }
  fModule = ::dlopen(fName.c_str(),flags);
  if(!fModule) {
#if defined(__APPLE__)
    std::string s = fName + ".bundle";
    fModule = ::dlopen(s.c_str(),flags);
    if(!fModule) {
      std::string s = "lib" + fName + ".dylib";
      fModule = ::dlopen(s.c_str(),flags);
    }
#else
    std::string s = "lib" + fName + ".so";
    fModule = ::dlopen(s.c_str(),flags);
#endif
  }
  if(!fModule) fError = ::dlerror();
#endif
#ifdef WIN32
  if(fVerbose) {
    std::cout << "AIDA::Loader::open :" 
              << " open \"" << fName << "\"..." 
              << std::endl;
  }
  fModule = ::LoadLibrary(fName.c_str());
#endif
  return fModule ? true : false;
}
//////////////////////////////////////////////////////////////////////////////
bool AIDA::Loader::isOpened(
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return fModule?true:false;
}
//////////////////////////////////////////////////////////////////////////////
const std::string& AIDA::Loader::fileName(
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return fName;
}
//////////////////////////////////////////////////////////////////////////////
AIDA::Procedure AIDA::Loader::find(
 const std::string& aName
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(!fModule) return 0;
  Procedure p = 0;
#ifdef DLOPEN
  const char* cerror;
  p = (Procedure)::dlsym(fModule,(char*)aName.c_str());
  if ((cerror=dlerror())!=NULL) { //error : undefined symbol.
    std::string serror = cerror;
    std::string s = "_";
    s += aName;
    p = (Procedure)::dlsym(fModule,(char*)s.c_str());
    if ((cerror=dlerror())!=NULL) { //error : undefined symbol.
      std::cerr << "AIDA::Loader::findProcedure :" 
                << " problem when searching \"" << aName << "\""
                << " in \"" << fName << "\"."
                << " dlerror  : " << serror 
                << std::endl;
    } 
  }
#endif
#ifdef WIN32
  p = (Procedure)::GetProcAddress((HMODULE)fModule,aName.c_str());
  if (!p) {
    std::string s = "_";
    s += aName;
    p = (Procedure)::GetProcAddress((HMODULE)fModule,s.c_str());
    if (!p) {
      std::cerr << "AIDA::Loader::findProcedure :"
                << " can't load " << aName << " procedure." 
                << std::endl;
    }
  }
#endif
  return p;
}


