//  Mai/2007 : G.Barrand : creation.

#ifndef SoSDL_Viewer_h
#define SoSDL_Viewer_h 

#include <Inventor/SDL/SoSDL.h>

#include <Inventor/SoType.h>

class SbVec2s;
class SbVec3f;
class SbColor;
class SbViewportRegion;

class SoSceneManager;
class SoNode;
class SoGLRenderAction;
class SoNodeSensor;
class SoSensor;
class SoEvent;
class SoGroup;
class SoCamera;
class SoDirectionalLight;

typedef void(*SoSDL_UnlockCB)(void*);

class SoSDL_Viewer {
public:
  SoSDL_Viewer(SoSDL_Widget* = 0);
  virtual ~SoSDL_Viewer();
  
  virtual SbBool setSceneGraph(SoNode*);
  virtual SoNode* getSceneGraph() const;
  virtual void setViewportRegion(const SbViewportRegion&);
  virtual const SbViewportRegion& getViewportRegion() const;
  virtual void setBackgroundColor(const SbColor&);
  virtual const SbColor& getBackgroundColor() const;
  virtual void render();
  virtual void setSize(const SbVec2s&);
  virtual void viewAll();
  virtual void setViewing(SbBool);
  virtual void show();
  virtual void setGLRenderAction(SoGLRenderAction*);
  virtual SoGLRenderAction* getGLRenderAction() const;
  virtual void setTitle(const char*);
  virtual void setCamera(SoCamera*);
  virtual SoCamera* getCamera() const;
  virtual void setCameraType(SoType);
  virtual SoType getCameraType(void) const;
  virtual void setHeadlight(SbBool);
  virtual void setAutoClipping(SbBool);
  virtual void setDecoration(SbBool);
  
  virtual SbBool processEvent(SDL_Event&,SbBool& render);
  virtual void lock(const SbString&,SoSDL_UnlockCB = 0,void* = 0); 

  void rotateCameraPositionAroundX(float);
  void rotateCameraPositionAroundY(float);
  void rotateCameraAroundUp(float);
  void translateCameraAlongDir(float);
  void translateCameraAlongSide(float);
  void translateCameraAlongUp(float);

  float getDefaultDeltaRotation() const;
  float getDefaultDeltaTranslation() const;
  void setDefaultDeltaRotation(float angle_radian);
  void setDefaultDeltaTranslation(float);

  float getFloor() const;
  void setFloor(float);
  void checkFloor(SbBool);

  SbBool emulateLeftButtonDown(SbBool&);
  SbBool emulateLeftButtonUp(SbBool&);
  SbBool emulateMouseMotion(SbBool&);

  enum Move {
    ROTATE_RIGHT,
    TRANSLATE_RIGHT,
    ROTATE_LEFT,
    TRANSLATE_LEFT,
    UP,
    DOWN,
    FORWARD,
    BACKWARD
  };
  void updateMotion(Move);
  void startMoveTimer();
  void stopMoveTimer();
  void setMoveTimerInterval(unsigned int interval_msec);

protected:
  virtual SbBool processEventLock(SDL_Event&,SbBool& render);
private:
  static Uint32 moveTimerCB(Uint32,void*);
private:
  void setCameraPosition(const SbVec3f&);
private:
  SoSceneManager* fSceneManager;
  SoGroup* fSceneGraph;
  SoNode* fUserSceneGraph;
  SoCamera* fCamera;
  SoDirectionalLight* fHeadLight;
  SoType fCameraType;
  float fDefaultDeltaRotation;
  float fDefaultDeltaTranslation;  
  SbBool fCheckFloor;
  float fFloor;
private:
  SbBool fJoy_rotate;
  SbBool fJoy_forback;
  Move fMove_what;
  Uint32 fMoveTimerInterval;
  SDL_TimerID fMoveTimerID;
  // lock system :
  SbString fPass;
  SbString fRead;
  SoSDL_UnlockCB fUnlockCB;
  void* fUnlockCB_Data;
};

#endif
